// EOFoundationExtras.h
// Copyright (c) 1995, NeXT Software, Inc. All rights reserved. 
//
// Non-EOF-specific extensions to foundation classes
//
#import	<Foundation/Foundation.h>
#import <Foundation/NSStringBuffer.h> //private header
#import <EOControl/EODefines.h>

@interface NSString (EOPrefixAndSuffixMethods)
- (NSString *)stringByDeletingSuffixWithDelimiter:(NSString *)delimiter;
- (NSString *)prefixWithDelimiter:(NSString *)delimiter;
- (NSString *)suffixWithDelimiter:(NSString *)delimiter;
- (NSString *)quotedStringWithQuote:(NSString *)quote;
    // places the quote character at the beginning and end of the string and
    // places a '\' before any quote character or '\' in the string.
- (BOOL)containsOnlyWhiteSpaceAndNewLines;
- (NSString *)stringRepeatedTimes:(unsigned)times;
    // [@"*" stringRepeatedTimes:5] -> "*****"
- (NSString *)stringMarkingUpcaseTransitionsWithDelimiter:(NSString *)delimiter;
    // returns string with delimeter inserted between every lower to uppercase
    // transition.  E.g., with Delimeter "_", "firstName" -> "first_Name"
@end

@interface NSString (PatternMatchingMethods)
- (NSString *)_getBracketedStringFromBuffer:(NSStringBuffer *)buf;
- (BOOL)_matchesCharacter:(unichar)sChar;
- (BOOL)matchesPattern:(NSString *)pattern;
    // Does shell type pattern matching using '*' and '?'
@end

@interface NSNumber (EOCustomValueMethods)
+ __numberWithString:(NSString *)string type:(NSString *)type;
@end

@interface NSString (EOCustomValueMethods)
- initWithString:(NSString *)string type:(NSString *)type;
- (NSString *)stringForType:(NSString *)type;
@end

@interface NSArray (EOArrayWithObjectsNotInArray)
- (NSArray *)arrayWithObjectsNotInArray:other;
- (NSArray *)indexesForObjectsIndenticalTo:(NSArray *)other;
- (NSArray *)objectsAtIndexes:(NSArray *)indexes;
    // given an array of NSNumbers, returns array of objects at given indexes
- (NSArray *)closestMatchingIndexesLessThan:(unsigned)max selectFirstOnNoMatch:(BOOL)selectFirst;
- (NSArray *)arrayWithValuesForKey:(NSString *)key;
    // maps valueForKeyPath across all items in array
- (unsigned)indexOfObjectMatchingValue:(id)value forKey:(NSString *)key;
    // returns index of first item in the array where valueForKeyPath:key
    // isEqual:value
- (unsigned)indexOfAttributeBySelector:(SEL)selector equalToObject:(id)candidate;
    // for each item, performs selector
    // NSNotFound iff not found
@end

@interface NSArray (EOArrayExtras)
- (NSDictionary *)_dictionaryWithNamedObjects;
    // Creates an autoreleased NSDictionary by taking each object in the array
    // and inserting it under the key [object name].
- _objectInSortedArrayWithName:(NSString *)name;
    // This assumes that self contains objects responding to -name and that
    // that those objects are sorted by name.
- (NSMutableArray *)resultsOfPerformingSelector:(SEL)aSelector;
    // sends aSelector to each object, returning the return values in an array
- (NSMutableArray *)shallowMutableCopy;
    // just copy the pointers. For NSArray, this is retain.
    // For mutable it is [[NSArray alloc] initWithArray:self]
@end

// Declares performSelector methods that don't return an id. Avoids the NSInvocation
// bug which retains return values from methods returning ids
@interface NSObject (EOObjectExtras)
- (unsigned)performSelectorReturningUnsigned:(SEL)selector;
- (unsigned)performSelectorReturningUnsigned:(SEL)selector withObject:(id)object;
- (unsigned)performSelectorReturningUnsigned:(SEL)selector withObject:(id)object1 withObject:(id)object2;
@end

@interface NSObject (EOPerformArray)
- (void)performSelector:(SEL)aSelector withEachObjectInArray:(NSArray *)array;
    // calls perform:withObject: on target with each object in array

- (NSArray *)resultsOfPerformingSelector:(SEL)aSelector withEachObjectInArray:(NSArray *)array;
    // calls perform:withObject: on target with each object in array and places
    // the result in result array

- (void)applyChangedArray:(NSArray *)anew old:(NSArray *)old addSelector:(SEL)addSel removeSelector:(SEL)removeSel;
    // calls addSelector for each object in new, but not old
    // and removeSelector for each in old but not in new.
@end

@interface NSDictionary (EODictionaryExtras)
- (BOOL)containsObjectsNotIdenticalTo:object;
    // checks whether the receiver contains any objects not
    // equal to any of the given object.  Pointer equality is used.
    // Used to scan for foreign key rows of all EONull.

- (BOOL)boolValueForKey:(NSString *)key default:(BOOL)yn;
    // Looks for value of "YES" or "NO".  If nil, returns default.
    // Useful for reading plist archives and defaults.

- (id)nilEnabledValueForKey:(NSString *)key;
    // Does a valueForKey: on self, and if the value is EONull, it returns nil.
    // Otherwise it returns the value.

- (NSMutableDictionary *)shallowMutableCopy;

- (NSDictionary *)changedValuesInDictionary:(NSDictionary *)newDictionary withKeys:(NSArray *)keys;

@end

@interface NSArray (EOPointerBasedManipulation)

- (NSArray *)arrayExcludingObjectsInArray:(NSArray *)array;
    // returns an array containing all objects in the receiver except for those objects
    // in "array"

- (NSArray *)arrayByRemovingObject:(id)obj;
    // This thing should be in Foundation !

- (BOOL)isIdentical:(NSArray *)other;
   // contains same object (id == id) in same order
   // (uses pointer equality for child comparison)

- (BOOL)_containsIdenticalObjectsInArray:(NSArray *)other;
    // Tests whether arrays are equal as sets (order independent)
    // Uses identity comparison (id == id).
    // DOES NOT WORK IF ARRAYS HAVE DUPLICATES.
    // This is really a set operation, so it's only here
    // for convenience. It's slow -- O(n2) and it doesn't
    // check for the number of times an object appears in
    // the array.  Used for EOModel attribute comparison.
@end


@interface NSMutableArray (EOPointerBasedManipulation)

- (BOOL)addObjectIfAbsent:object;
    // adds an object to the array if only if it isn't already in the array

@end


@interface NSMutableDictionary (EOPointerBasedManipulation)

- (void)translateFromKeys:(NSArray *)currentKeys toKeys:(NSArray *)newKeys;
    // changes the keys in a dictionary

- (void)overrideEntriesWithObjectsFromDictionary:(NSDictionary *)other keys:(NSArray *)keys;

- (void)removeKeysNotIn:(NSArray *)keysToKeep;

@end

@interface NSString (EOPointerBasedManipulation)

- (NSString *)relationshipPathByDeletingLastComponent;
    // rel1.rel2.rel3 --> rel1.rel2

- (NSString *)lastComponentFromRelationshipPath;
    // rel1.rel2.rel3 --> rel3

- (NSString *)relationshipPathByDeletingFirstComponent;
    // rel1.rel2.rel3 --> rel2.rel3

- (NSString *)firstComponentFromRelationshipPath;
    // rel1.rel2.rel3 --> rel1

- (BOOL)relationshipPathIsMultiHop;
    // returns YES if self contains a period.
@end


@interface NSBundle (EOLibraryPathSeach)
+ (NSString *)pathForLibraryResource:(NSString *)name type:(NSString *)ext directory: (NSString *)directory;
    // Used for finding models.  If directory == "Models", will search in:
    // AppWrapper/Resources, AppWrapper, ~/Library/Models,
    // /LocalLibrary/Models, /NextLibrary/Models
@end

@interface NSCalendarDate (EOMillisecondManipulation)
- (void)setMilliseconds:(unsigned)milliseconds;
    // millisecond value should be between 0 and 999
@end

// macros to make class checks very fast
typedef struct {@defs(NSObject)} _NSObjectStruct;
#define objectIsOfClass(object, class) (((_NSObjectStruct  *)object)->isa == (class))
#define objectsAreOfSameClass(a, b) (((_NSObjectStruct  *)a)->isa == ((_NSObjectStruct  *)b)->isa)
#define classOfObject(object) ((Class)((_NSObjectStruct  *)object)->isa)

EOCONTROL_EXTERN unichar _EOFNextCharacterInPattern(NSStringBuffer *buf, BOOL *isMagic);

