/*
 * JavaBridge -- java-objc.h
 *
 * Copyright (c) 1996-1997, Apple Computer, Inc.
 * All Rights Reserved.
 */

#ifndef JAVA_OBJC_H
#define JAVA_OBJC_H

#if defined(WIN32)
    #if defined(NSBUILDINGJOBJC)
        #define JOBJC_EXPORT __declspec(dllexport)
    #else
        #define JOBJC_EXPORT __declspec(dllimport)
    #endif
#else
    #define JOBJC_EXPORT extern
#endif

#import <Foundation/Foundation.h>
#import <ObjCJava/vm-interface.h>
#import <ObjCJava/typemap.h>

/*
 * Create a java object wrapper around an Objc object.  If a java class has
 * been associated with the Objc class (using NSSetJavaClassForObjcClass()),
 * then an instance of that class is created to wrap the Objc object.  If
 * not, the Objc superclass hierarchy is searched for a matching java wrapper
 * class and an instance of the closest match is created.
 *
 * For example, consider an Objc class Foo which is mapped to the Java class
 * JavaFoo, and Bar, a subclass of Foo, which is not mapped to any Java class.
 * Calling NSCreateJavaWrapper() on an instance of Bar would create a JavaFoo
 * instance to wrap the Bar instance.
 *
 * In order to prevent a race condition with the garbage collector, we ensure
 * that the Java object cannot be finalized as long as the Objc object is
 * being referenced in the Objc world.  We do this by creating a global reference
 * to the Java object and only releasing that reference when nothing in the
 * Objc world refers to the Objc object.  At this point, the Java object will
 * still be referencing the Objc object, but it is now able to be finalized.
 * When the Java object is finalized, it will release the Objc object which will
 * (finally) be dealloc'd.
 *
 * This has two consequences you should be aware of: (1) Objc objects that are
 * wrapped will not be dealloc'd until the Java object is finalized, and (2) we
 * use some grody runtime hackery to replace the retain/release methods on the
 * class of the Objc object that is being wrapped.  This means that retain/release
 * on all instances of that class will be slightly slower.
 *
 * Note also that this function is extremely very not thread safe!
 */
JOBJC_EXPORT JAVAHandle NSCreateJavaWrapper (id objcObject);

/*
 * Create a real-live java object.  The signature specifies the constructor
 * to use when instantiating the class; passing in NULL uses the default
 * constructor.  If a java exception is thrown while creating the object,
 * an NSException is thrown which encapsulates the java exception.
 *
 * The java class name must be specified using '/' to separate the package
 * name components, not '.'.
 *
 * Note that although it looks like you can actually pass arguments to the
 * constructor, and in actual fact you can on i386 platforms, if you try
 * doing this on any other platform, it will blow up in your face.  Any
 * sparc assembly hackers out there?
 *
 * This function is more than just a convenience around the JAVANewObject()
 * function since it also converts any ObjC object arguments to Java objects.
 */
JOBJC_EXPORT id NSCreateJavaObject (const char *javaClassName, const char *signature, ...);
JOBJC_EXPORT id NSCreateJavaObjectV (const char *javaClassName, const char *signature, BOOL shortFloatsOnStack, va_list args);

/*
 * Lookup an object's counterpart in the other world.  If a counterpart
 * doesn't exist, one is created on the fly.  By default, Objc objects are
 * wrapped in java objects using the NSCreateJavaWrapper() function and java
 * objects are wrapped in an NSJavaObject proxy.  If a mapping function
 * exists for the class (specified using the NSSetJavaMappings() function),
 * it will be used to create the counterpart instead.
 */
JOBJC_EXPORT JAVAHandle NSIdToJavaHandle (id objcObject);
JOBJC_EXPORT id NSJavaHandleToId (JAVAHandle javaHandle);

/*
 * Associate a java class with an Objc class.  Whenever an instance of the
 * specified Objc class is passed across the bridge, it will be wrapped in
 * an instance of the associated java class.  The java class name must be
 * specified using '/' to separate the package name components, not '.'.
 *
 * The java class name is specified as a string rather than a JAVAClass
 * handle so we don't have to go resolving all classes just because an
 * association has been created.  This would blow out of the water the VM's
 * lazy-loading capabilities.
 *
 * To prevent the VM blowing up in your face, the java class *must* be a
 * subclass of next/util/NextObject.  Bad stuff will happen otherwise, I
 * promise.
 */
JOBJC_EXPORT void NSSetJavaClassForObjcClass (char *javaClassName, Class objcClass);

/*
 * Associate a java method with an Objc method.  This is used to map multi-
 * component selector names to java method names and back.
 */
JOBJC_EXPORT void NSSetJavaMethodForObjcSelector (char *javaMethodName, SEL objcSelector);

/*
 * Specify functions which should be used to map Objc objects to java objects
 * and vice versa.
 */
typedef void *(NSJavaClassMapFunction)(void *o);

JOBJC_EXPORT void NSSetJavaClassMappings (char *javaClassName, char *objcClassName, NSJavaClassMapFunction *javaToObjc, NSJavaClassMapFunction *objcToJava);

/*
 * Specify functions which should be used to map Objc types, such as structs,
 * to Java objects.  If the stackSize pointer is specified, the conversion
 * function must return the number of bytes occupied by this struct if it
 * were passed byvalue on the stack.
 */
typedef JAVAHandle (NSJavaTypeMapFunction) (void *objcType, int *stackSize);

JOBJC_EXPORT void NSSetJavaTypeMappings (char *javaClassName, NSJavaTypeMapFunction *objcToJava);

#endif JAVA_OBJC_H
